package com.engine.sskj.service.impl;

import cn.hutool.core.date.DateUtil;
import com.engine.core.impl.Service;
import com.engine.sskj.service.EntryService;
import com.engine.sskj.util.SskjUtil;
import org.apache.commons.lang.StringUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.Util;

import java.util.*;

public class EntryServiceImpl extends Service implements EntryService {

    BaseBean basebean = new BaseBean();
    @Override
    public Map<String, Object> checkBlackList(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>();

        RecordSet rs = new RecordSet();

        String idNumbers = Util.null2String(params.get("idNumbers"));//获取入参中的身份证号

        if ( StringUtils.isBlank(idNumbers)) {
            result.put("code",500);
            result.put("msg","未获取到身份证号。");
            return result;
        }

        List<String> inList = new ArrayList<>();
        String[] idNumbersArr = idNumbers.split(",");
        for (String idNumber:idNumbersArr) {
            Integer number = -1;
            String checkSql = "select count(*) as number from uf_hmd where sfzh = ? ";
            rs.executeQuery(checkSql,idNumber);
            while (rs.next()) {
                number = Util.getIntValue(Util.null2String(rs.getString("number")));
            }
            if ( number > 0 ) {//在黑名单中
                inList.add(idNumber);
            }
        }

        if ( !inList.isEmpty()) {
            String inStr = String.join(",",inList);
            result.put("code",500);
            result.put("msg","身份证号:"+inStr+" 在黑名单中，是否继续入职申请？");
            return result;
        }

        result.put("code",200);
        result.put("msg","该批人员身份证号均不在黑名单中。");
        return result;
    }

    @Override
    public Map<String, Object> checkEntryStatus(Map<String, Object> params) {
        Map<String, Object> result = new HashMap<>();
        RecordSet rs = new RecordSet();

        String dataIds = Util.null2String(params.get("dataIds"));

        if (StringUtils.isEmpty(dataIds)) {
            result.put("code",500);
            result.put("msg","未选择需要检测入职状态的员工");
            return result;
        }

        String[] idsArr = dataIds.split(",");

        List<String> errorStatusList = new ArrayList<>();
        List<String> errorPromptList = new ArrayList<>();

        String checkStatusSql = "select rzzt,xm from uf_rzgl where id = ?";
        for (String id: idsArr) {
            rs.executeQuery(checkStatusSql,id);
            while (rs.next()) {
                Integer status = Util.getIntValue(Util.null2String(rs.getString("rzzt")));
                String lastname = Util.null2String(rs.getString("xm"));
                if (status != 0 ) {
                    errorStatusList.add(id);
                    errorPromptList.add(lastname);
                }
            }
        }

        if ( !errorStatusList.isEmpty()) {//存在不为待入职状态的数据
            String prompStr = String.join(",",errorPromptList);
            result.put("code",500);
            result.put("msg","姓名为："+prompStr+" 的人员，入职状态不出于待入职状态，不可以办理入职。");//确认一下需要提示什么
            return result;
        }

        result.put("code",200);
        result.put("msg","均可办理入职");
        return result;
    }

    @Override
    public Map<String, Object> grantNumber(Map<String, Object> params) {

        Map<String, Object> result = new HashMap<>();
        RecordSet rs = new RecordSet();

        String serviceCompany = Util.null2String(params.get("serviceCompany"));
        String entryType = Util.null2String(params.get("entryType"));

        if ( StringUtils.isEmpty(entryType)) {
            result.put("code",500);
            result.put("msg","入职类型未确认，不可分发编号");
            return result;
        }

        if ( "6".equals(entryType) && StringUtils.isEmpty(serviceCompany)) {
            result.put("code",500);
            result.put("msg","请先选择劳务公司");
            return result;
        }

        if ( "6".equals(entryType)) {//劳务外包
            //获取劳务公司代码
            String compCode = "";
            String acqOutCompSql = "select gsdm from uf_lwgs where id = ? ";
            rs.executeQuery(acqOutCompSql, serviceCompany);
            while (rs.next()) {
                compCode = Util.null2String(rs.getString("gsdm"));
                if ( StringUtils.isBlank(compCode)) {
                    result.put("code",500);
                    result.put("msg","请先维护劳务公司代码");
                    return result;
                }
            }
            //获取该劳务公司最大流水号
            String maxNum = "";
            String entryMonth = DateUtil.format(new Date(),"yyyyMM");
            String acqMaxNumSql = "select maxNum from uf_OutEmpNum where outComp = ? and entryDate = ?";
            rs.executeQuery(acqMaxNumSql, serviceCompany, entryMonth);
            while (rs.next()) {
                maxNum = Util.null2String( rs.getString("maxNum"));
            }
            if ( StringUtils.isBlank(maxNum)) {//该公司该月没有生成最大流程号
                SskjUtil sskjUtil =new SskjUtil();
                maxNum = "001";
                String addMaxNumSql = "insert into uf_OutEmpNum ( outComp, entryDate, maxNum, formmodeid, modedatacreater, modedatacreatertype, modedatacreatedate, modedatacreatetime, MODEUUID)" +
                        " values (?,?,?,?,?,?,?,?,?)";
                String outEmpNumModeId = basebean.getPropValue("project_sskj", "outEmpNumModeId");
                Integer modedatacreater = user.getUID();
                Integer modedatacreatertype = user.getType();
                String modedatacreatedate = DateUtil.format(new Date(), "yyyy-MM-dd");
                String modedatacreatetime = DateUtil.format(new Date(), "HH:mm:ss");
                String uuid = UUID.randomUUID().toString();

                boolean addFlag = rs.executeUpdate(addMaxNumSql, serviceCompany, entryMonth, maxNum, outEmpNumModeId, modedatacreater, modedatacreatertype, modedatacreatedate,
                        modedatacreatetime, uuid);
                if (addFlag) {
                    String billid = "-1";
                    String acqModeIdSql = "select id from uf_OutEmpNum where MODEUUID = ?";
                    rs.executeQuery(acqModeIdSql, uuid);
                    while (rs.next()) {
                        billid = Util.null2String(rs.getString("id"));
                    }
                    sskjUtil.modePerRecon(modedatacreater, outEmpNumModeId, billid);
                }
                result.put("code",200);
                result.put("data",compCode + entryMonth + maxNum);
                return result;
            } else {
                Integer maxNumInt = Integer.valueOf(maxNum);
                maxNumInt = maxNumInt + 1;
                maxNum = String.valueOf(maxNumInt);
                if (maxNum.length() == 1 ) {
                    maxNum = "00" + maxNum;
                }
                if (maxNum.length() == 2 ) {
                    maxNum = "0" + maxNum;
                }
                String modMaxNumSql = "update uf_OutEmpNum set maxNum = ? where outComp = ? and entryDate = ?";
                rs.executeUpdate(modMaxNumSql, maxNum, serviceCompany, entryMonth);
                result.put("code",200);
                result.put("data",compCode + entryMonth + maxNum);
                return result;
            }

        } else {//非劳务外包
            //获取最大流水号
            String maxNum = "";
            String entryMonth = DateUtil.format(new Date(),"yyyyMM");
            String acqMaxNumSql = "select maxNum from uf_NoOutEmpNum where entryDate = ?";
            rs.executeQuery(acqMaxNumSql, entryMonth);
            while (rs.next()) {
                maxNum = Util.null2String( rs.getString("maxNum"));
            }
            basebean.writeLog("maxNum：" + maxNum);
            if ( StringUtils.isBlank(maxNum)) {//该月没有生成最大流程号
                SskjUtil sskjUtil =new SskjUtil();
                maxNum = "001";
                basebean.writeLog("非外包，没有流水");
                basebean.writeLog("maxNum：" + maxNum);

                String addMaxNumSql = "insert into uf_NoOutEmpNum ( entryDate, maxNum, formmodeid, modedatacreater, modedatacreatertype, modedatacreatedate, modedatacreatetime, MODEUUID)" +
                        " values (?,?,?,?,?,?,?,?)";
                String noOutEmpNumModeId = basebean.getPropValue("project_sskj", "noOutEmpNumModeId");
                Integer modedatacreater = user.getUID();
                Integer modedatacreatertype = user.getType();
                String modedatacreatedate = DateUtil.format(new Date(), "yyyy-MM-dd");
                String modedatacreatetime = DateUtil.format(new Date(), "HH:mm:ss");
                String uuid = UUID.randomUUID().toString();
                basebean.writeLog("addMaxNumSql：" + addMaxNumSql);

                boolean addFlag = rs.executeUpdate(addMaxNumSql, entryMonth, maxNum, noOutEmpNumModeId, modedatacreater, modedatacreatertype, modedatacreatedate,
                        modedatacreatetime, uuid);
                basebean.writeLog("addFlag：" + addFlag);
                if (addFlag) {
                    String billid = "-1";
                    String acqModeIdSql = "select id from uf_NoOutEmpNum where MODEUUID = ?";
                    rs.executeQuery(acqModeIdSql, uuid);
                    while (rs.next()) {
                        billid = Util.null2String(rs.getString("id"));
                    }
                    basebean.writeLog("billid：" + billid);
                    sskjUtil.modePerRecon(modedatacreater, noOutEmpNumModeId, billid);
                }
                result.put("code",200);
                result.put("data","SSKJ" + entryMonth + maxNum);
                return result;
            } else {
                Integer maxNumInt = Integer.valueOf(maxNum);
                basebean.writeLog("maxNumInt：" + maxNumInt);
                maxNumInt = maxNumInt + 1;
                basebean.writeLog("maxNumInt：" + maxNumInt);
                maxNum = String.valueOf(maxNumInt);
                basebean.writeLog("maxNum：" + maxNum);
                if (maxNum.length() == 1 ) {
                    maxNum = "00" + maxNum;
                }
                if (maxNum.length() == 2 ) {
                    maxNum = "0" + maxNum;
                }
                basebean.writeLog("非外包，已有流水");
                basebean.writeLog("maxNum：" + maxNum);
                String modMaxNumSql = "update uf_NoOutEmpNum set maxNum = ? where entryDate = ?";
                boolean b = rs.executeUpdate(modMaxNumSql, maxNum, entryMonth);
                basebean.writeLog("b：" + b);
                result.put("code",200);
                result.put("data","SSKJ" + entryMonth + maxNum);
                return result;
            }
        }
    }
}
