package weaver.interfaces.sskj.util;




import org.apache.commons.collections.MapUtils;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.StatusLine;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.*;
import org.apache.http.config.Registry;
import org.apache.http.config.RegistryBuilder;
import org.apache.http.conn.HttpClientConnectionManager;
import org.apache.http.conn.socket.ConnectionSocketFactory;
import org.apache.http.conn.socket.PlainConnectionSocketFactory;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.io.IOException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.*;


public class HttpsUtil {

    private static CloseableHttpClient httpclient = HttpClientBuilder.create().setConnectionManager(poolingConnectionManager()).build();

    public static HttpClientConnectionManager poolingConnectionManager() {
        PoolingHttpClientConnectionManager poolingConnectionManager = new PoolingHttpClientConnectionManager();
        // 连接池最大连接数
        poolingConnectionManager.setMaxTotal(1000);
        // 每个主机的并发
        poolingConnectionManager.setDefaultMaxPerRoute(100);
        return poolingConnectionManager;
    }

    private static RequestConfig requestConfig = RequestConfig.custom()
            .setConnectTimeout(1000)
            // 从连接池中获取到连接的最长时间
            .setConnectionRequestTimeout(500)
            // 数据传输的最长时间,联合差补需要同步较长时间
            .setSocketTimeout(60000)
            .build();


    private static void prepareHeaders(HttpRequestBase httpRequest, Map<String, String> headers) {
        if (!MapUtils.isEmpty(headers)) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                httpRequest.addHeader(entry.getKey(), entry.getValue());
            }
        }
    }

    private static String sendRequest(HttpRequestBase httpRequest) throws IOException {
        String result = null;
        CloseableHttpClient httpclient = creteClient();
        try (CloseableHttpResponse resp = httpclient.execute(httpRequest)) {
            HttpEntity entity = resp.getEntity();
            result = (entity == null ? null : EntityUtils.toString(entity, Consts.UTF_8));
            StatusLine statusLine = resp.getStatusLine();
            if (statusLine.getStatusCode() >= 300) {
                EntityUtils.consume(entity);
//                String errorMsg = Strings.lenientFormat("Http status abnormal-status code is %s, response is %s", resp.getStatusLine().getStatusCode(), result);
            }
        }
        return result;
    }

    public static String doPostJsonRequest(String url, Map<String, String> headers, String jsonStr) throws IOException {
        HttpPost httpost = new HttpPost(url);
        prepareHeaders(httpost, headers);
        StringEntity entity = new StringEntity(jsonStr, Consts.UTF_8);
//        entity.setContentEncoding("UTF-8");
        entity.setContentType("application/json");
        httpost.setEntity(entity);
        return sendRequest(httpost);
    }

    public static String doGet(String url, Map<String, String> headers) throws IOException {
        HttpGet httpget = new HttpGet(url);
        prepareHeaders(httpget, headers);
        return sendRequest(httpget);
    }

    public static String doPost(String url, Map<String, String> headers, Map<String, String> params) throws IOException {
        HttpPost httpost = new HttpPost(url);
        prepareHeaders(httpost, headers);
        //组织请求参数
        List<NameValuePair> paramList = new ArrayList<NameValuePair>();
        if (params != null && params.size() > 0) {
            Set<String> keySet = params.keySet();
            for (String key : keySet) {
                paramList.add(new BasicNameValuePair(key, params.get(key)));
            }
        }
        UrlEncodedFormEntity entity = new UrlEncodedFormEntity((Iterable<? extends NameValuePair>) paramList, Consts.UTF_8);
        entity.setContentType("application/x-www-form-urlencoded");
        httpost.setEntity(entity);
        return sendRequest(httpost);
    }


    public static byte[] doGetWithByteArrayResp(String url, Map<String, String> headers) throws IOException {
        HttpGet httpGet = new HttpGet(url);
        httpGet.setConfig(requestConfig);
        prepareHeaders(httpGet, headers);
        return sendRequestWithByteArrayResp(httpGet);
    }



    private static byte[] sendRequestWithByteArrayResp(HttpRequestBase httpRequest) throws IOException {
        httpRequest.addHeader(HTTP.CONTENT_ENCODING, "UTF-8");
        byte[] result = null;
        CloseableHttpClient httpclient = creteClient();
        try (CloseableHttpResponse resp = httpclient.execute(httpRequest)) {
            HttpEntity entity = resp.getEntity();
            result = (entity == null ? null : EntityUtils.toByteArray(entity));
//            StatusLine statusLine = resp.getStatusLine();
//            if (statusLine.getStatusCode() >= 300) {
//                EntityUtils.consume(entity);
//                String errorMsg = Strings.lenientFormat("Http status abnormal-status code is %s, response is %s", resp.getStatusLine().getStatusCode(), result);
//                throw new HttpResponseException(statusLine.getStatusCode(), statusLine.getReasonPhrase());
//            }
        }
        return result;
    }


    /**
     * 文件上传
     *
     * @param url             请求路径
     * @param headers         请求头部
     * @param params          请求参数
     * @param files           附件
     * @param requestFileName 附件接受名称
     * @return
     * @throws IOException
     */
    public static String doPostFormMultipart(String url, Map<String, String> headers, Map<String, String> params, List<MultipartFile> files, String requestFileName) throws IOException {
        HttpPost httpost = new HttpPost(url);
        prepareHeaders(httpost, headers);
        //组织请求参数
        if (params != null && params.size() > 0) {
            List<NameValuePair> paramList = new ArrayList<NameValuePair>();
            Set<String> keySet = params.keySet();
            for (String key : keySet) {
                paramList.add(new BasicNameValuePair(key, params.get(key)));
            }
        }
        MultipartEntityBuilder mEntityBuilder = MultipartEntityBuilder.create();
        mEntityBuilder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
        mEntityBuilder.setCharset(Consts.UTF_8);

        // 普通参数
        ContentType contentType = ContentType.create("text/plain", Consts.UTF_8);//解决中文乱码
        if (params != null && params.size() > 0) {
            Set<String> keySet = params.keySet();
            for (String key : keySet) {
                mEntityBuilder.addTextBody(key, params.get(key), contentType);
            }
        }
        //二进制参数
        if (files != null && files.size() > 0) {
            for (MultipartFile file : files) {
                mEntityBuilder.addBinaryBody(requestFileName, file.getInputStream(), ContentType.create("multipart/form-data", Consts.UTF_8), file.getName());
            }
        }
        httpost.setEntity(mEntityBuilder.build());
        return sendRequest(httpost);
    }


    public static String doPutJsonRequest(String url, Map<String, String> headers, String jsonStr) throws IOException {
        HttpPut httpPut = new HttpPut(url);
        prepareHeaders(httpPut, headers);
        StringEntity entity = new StringEntity(jsonStr, Consts.UTF_8);
        entity.setContentType("application/json");
        httpPut.setEntity(entity);
        return sendRequest(httpPut);
    }


    public static String doDelete(String url, Map<String, String> headers) throws IOException {
        HttpDelete httpDelete = new HttpDelete(url);
        prepareHeaders(httpDelete, headers);
        return sendRequest(httpDelete);

    }


    /**
     * 绕过验证
     *
     * @return
     * @throws NoSuchAlgorithmException
     * @throws KeyManagementException
     */
    public static SSLContext createIgnoreVerifySSL() throws NoSuchAlgorithmException, KeyManagementException {
        SSLContext sc = SSLContext.getInstance("SSLv3");

        // 实现一个X509TrustManager接口，用于绕过验证，不用修改里面的方法
        X509TrustManager trustManager = new X509TrustManager() {
            @Override
            public void checkClientTrusted(
                    java.security.cert.X509Certificate[] paramArrayOfX509Certificate,
                    String paramString) throws CertificateException {
            }

            @Override
            public void checkServerTrusted(
                    java.security.cert.X509Certificate[] paramArrayOfX509Certificate,
                    String paramString) throws CertificateException {
            }

            @Override
            public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                return null;
            }
        };

        sc.init(null, new TrustManager[]{trustManager}, null);
        return sc;
    }


    public static CloseableHttpClient creteClient() {
        //采用绕过验证的方式处理https请求
        try {
            SSLContext sslcontext = createIgnoreVerifySSL();
            // 设置协议http和https对应的处理socket链接工厂的对象
            Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory>create()
                    .register("http", PlainConnectionSocketFactory.INSTANCE)
                    .register("https", new SSLConnectionSocketFactory(sslcontext, NoopHostnameVerifier.INSTANCE))
                    .build();
            PoolingHttpClientConnectionManager connManager = new PoolingHttpClientConnectionManager(socketFactoryRegistry);
            HttpClients.custom().setConnectionManager(connManager);

            //创建自定义的httpclient对象
            CloseableHttpClient client = HttpClients.custom().setConnectionManager(connManager).build();
            return client;
        } catch (Exception e) {
            return null;
        }

    }


}
